(* Drucken von mehreren EPS-Dateien auf einer Seite

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 2.5 - Dez. 2005
         2.6 - Jan. 2006: Vorschau,
                          zu ignorierende PS-Befehle sind in Ini-Datei einstellbar
         2.7 - Dez. 2006: neues Unterroutinenpaket
         2.8 - Apr. 2010: Aktualisierung der Units
   letzte nderung: Mrz 2022
   *)

unit PrepsMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.ComCtrls, Vcl.Menus,
  Vcl.Buttons, Vcl.StdCtrls, System.IniFiles, WinUtils,
  WinPlot, WinShell, HListBox, ExtStreams, PSFontDlg, FileFilterDlg, SizeDlg,
  NumDlg, PsPrintDlg;

const
  Prog = 'Drucken von EPS-Dateien';
  Vers = '3.3';
  CopRgt = ' 2016-2022 Dr. J. Rathlev, D-24222 Schwentinental';

  BExt = 'pps';
  PExt = 'ps';

  BufSize = 128*1024;

type
  TPaperFormat = (piUser,piA4,piA3,piA2,piA1,piA0);

const
  PaperFormNames : array [piUser..piA0] of string =
          ('var.','A4','A3','A2','A1','A0');

  PapA4 : TFPoint = (x : 21.0; y : 29.6);      (* Standard-Papiergr. *)
  PapA3 : TFPoint = (x : 29.6; y : 42.0);
  PapA2 : TFPoint = (x : 42.0; y : 59.2);
  PapA1 : TFPoint = (x : 59.2; y : 84.0);
  PapA0 : TFPoint = (x : 84.0; y : 118.4);

  DefSize : TFPoint = (x : 10.0; y : 10.0);    (* Standard-Bounding-Box *)

  DefFootBottom = 0.5;               (* Position der Funote *)
  DefFootLeft = 2.0;
  DefRand : TFRect = (Left : 0.75; Top : 0.25; Right : 0.25; Bottom : 0.75);
  DefRaster = 5;                 (* 5 mm *)
  DefLPT = 3;
  DefFontTyp = 1;
  DefFontSize = 18;
  defName = 'Preps.ps';
  defFilter ='EPS-Dateien|*.eps|alle|*.*';

  ToolButtonPos : array [0..14] of word =
            (1,31,61,96,126,156,191,221,256,286,316,346,381,411,446);

  CmToPt = 28.35;                     (* pt / cm *)

  DefScaleFak = 64.0;
  FrameCol = clYellow;

type
  TBoxTyp = (btDefault,btBoundingBox,btText);
  TEPSBox = class (TObject)
    constructor Create (BoxSize : TFPoint);
    procedure Assign (ABox : TEPSBox);
  private
    PSBoxName : string;
    BoxTyp    : TBoxTyp;
    Orig,Offs,Size : TFPoint;
    BFont     : TPSFont;
    end;

  TLineal = record
    Schrift,Abstand,Grob,Fein,Skala : double;
    end;

  TPrepsForm = class(TForm)
    MainMenu: TMainMenu;
    Einstellungen1: TMenuItem;
    EpsPfadItem: TMenuItem;
    FilterItem: TMenuItem;
    PapierFormat1: TMenuItem;
    BoxsizeItem: TMenuItem;
    N3: TMenuItem;
    PrintSelectItem: TMenuItem;
    HochItem: TMenuItem;
    QuerItem: TMenuItem;
    Info: TMenuItem;
    ToolPanel: TPanel;
    StatusBar: TStatusBar;
    GrafikPanel: TPanel;
    GrafikFeld: TPaintBox;
    VertScrollBar: TScrollBar;
    HorzScrollBar: TScrollBar;
    EndBtn: TSpeedButton;
    PrintBtn: TSpeedButton;
    ClearBtn: TSpeedButton;
    OpenDialog: TOpenDialog;
    N4: TMenuItem;
    A4FormItem: TMenuItem;
    IndiItem: TMenuItem;
    RasterItem: TMenuItem;
    N6: TMenuItem;
    LPT11: TMenuItem;
    LPT21: TMenuItem;
    LPT31: TMenuItem;
    LPT41: TMenuItem;
    HochBtn: TSpeedButton;
    QuerBtn: TSpeedButton;
    PageBtn: TSpeedButton;
    Datei1: TMenuItem;
    Drucken1: TMenuItem;
    Anzeige1: TMenuItem;
    anpassen1: TMenuItem;
    grer1: TMenuItem;
    kleiner1: TMenuItem;
    neuesBlatt1: TMenuItem;
    LoadItem: TMenuItem;
    SaveItem: TMenuItem;
    N1: TMenuItem;
    EPSeinfgen1: TMenuItem;
    N2: TMenuItem;
    EndeItem: TMenuItem;
    SaveDialog: TSaveDialog;
    SaveAsItem: TMenuItem;
    N7: TMenuItem;
    DateiListe: TMenuItem;
    Text1: TMenuItem;
    InsertTextItem: TMenuItem;
    N8: TMenuItem;
    PSFontItem: TMenuItem;
    TextPopup: TPopupMenu;
    Text2: TMenuItem;
    N9: TMenuItem;
    DeltextItem: TMenuItem;
    EdiTextItem: TMenuItem;
    TextFontItem: TMenuItem;
    CopyTextItem: TMenuItem;
    N10: TMenuItem;
    RasterWeite: TMenuItem;
    RasterOn: TMenuItem;
    Aktualisieren1: TMenuItem;
    FootItem: TMenuItem;
    EpsPopup: TPopupMenu;
    Bild1: TMenuItem;
    N11: TMenuItem;
    EpsReplItem: TMenuItem;
    EpsUpdateItem: TMenuItem;
    EpsDelItem: TMenuItem;
    PrintFootItem: TMenuItem;
    FootPosItem: TMenuItem;
    N12: TMenuItem;
    A3FormItem: TMenuItem;
    A2FormItem: TMenuItem;
    A1FormItem: TMenuItem;
    A0FormItem: TMenuItem;
    LoadBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    InsertBtn: TSpeedButton;
    UpdateBtn: TSpeedButton;
    TextBtn: TSpeedButton;
    ZoomIn: TSpeedButton;
    ZoomOut: TSpeedButton;
    RasterBtn: TSpeedButton;
    N5: TMenuItem;
    Werkzeugleiste1: TMenuItem;
    TBTopItem: TMenuItem;
    TBLeftItem: TMenuItem;
    PreviewBtn: TSpeedButton;
    PSOutputItem: TMenuItem;
    OutEpsDirItem: TMenuItem;
    OutDirItem: TMenuItem;
    SuffixItem: TMenuItem;
    procedure InsertBtnClick(Sender: TObject);
    procedure EpsPfadItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EndBtnClick(Sender: TObject);
    procedure FilterItemClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure LargerItemClick(Sender: TObject);
    procedure SmallerItemClick(Sender: TObject);
    procedure FitItemClick(Sender: TObject);
    procedure GrafikFeldPaint(Sender: TObject);
    procedure InfoClick(Sender: TObject);
    procedure GrafikFeldMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure GrafikFeldMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ClearBtnClick(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure LPT11Click(Sender: TObject);
    procedure LPT21Click(Sender: TObject);
    procedure LPT31Click(Sender: TObject);
    procedure LPT41Click(Sender: TObject);
    procedure A4FormItemClick(Sender: TObject);
    procedure QuerItemClick(Sender: TObject);
    procedure HochItemClick(Sender: TObject);
    procedure IndiItemClick(Sender: TObject);
    procedure BoxsizeItemClick(Sender: TObject);
    procedure RasterItemClick(Sender: TObject);
    procedure LoadBtnClick(Sender: TObject);
    procedure SaveAsBtnClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure LoadDataListClick (Sender     : TObject;
                                 FileName   : string);
    procedure UpdateBtnClick(Sender: TObject);
    procedure InsertTextItemClick(Sender: TObject);
    procedure PSFontItemClick(Sender: TObject);
    procedure DelTextItemClick(Sender: TObject);
    procedure TextFontItemClick(Sender: TObject);
    procedure EditTextItemClick(Sender: TObject);
    procedure CopyTextItemClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RasterBtnClick(Sender: TObject);
    procedure RasterOnClick(Sender: TObject);
    procedure FootPosItemClick(Sender: TObject);
    procedure EpsDelItemClick(Sender: TObject);
    procedure EpsUpdateItemClick(Sender: TObject);
    procedure EpsReplItemClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PrintFootItemClick(Sender: TObject);
    procedure A3FormItemClick(Sender: TObject);
    procedure A2FormItemClick(Sender: TObject);
    procedure A1FormItemClick(Sender: TObject);
    procedure A0FormItemClick(Sender: TObject);
    procedure TBTopItemClick(Sender: TObject);
    procedure TBLeftItemClick(Sender: TObject);
    procedure StatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure PreviewBtnClick(Sender: TObject);
    procedure OutEpsDirItemClick(Sender: TObject);
    procedure OutDirItemClick(Sender: TObject);
    procedure SuffixItemClick(Sender: TObject);
  private
    { Private-Deklarationen }
    PrgPfad,PrgName,
    ArbPfad,FName,
    Ininame,ProgPfad,
    PrtFile,PrtPfad,
    BName,Textline,
    ProgVers,PsIgnore,
    Suffix,
    EpsFilter      : string;
    TextFont       : TPSFont;
    FileList       : THistoryList;
    FormPos        : TRect;
    TextCopy,NewBox,
    FootNote,
    SizeSet,chg,
    QuerFormat     : boolean;
    GrPlot         : TPlot;
    GrSize         : TPoint;   { Bildschirmgrafikfenster in Pixel }
    Usegrid        : boolean;
    PaperFormat    : TPaperFormat;
    Raster,PrtNr,
    MNr,MRaster,
    pn             : integer;
    WPos,         
    MPos           : TPoint;
    ScaleFak       : double;
    GScale,OldOrg,
    BoxSize,
    UserSize,
    PapSize        : TFPoint;
    URand          : TFRect;   { Rnder in cm/inch }
    PRand          : TRect;
    FootPosX,
    FootPosY,
    XYRatio        : double;
    EPSList        : TStringList;
    FramePos       : TRect;
    FrameOn        : boolean;
    FootBox        : TEPSBox;
    PrintMode      : TPrintMode;
    Lineal         : TLineal;

    procedure SetPaperSize (AIndex : TPaperFormat);
    procedure InitScale (ASize : TFPoint);
    procedure ChangePaperSize (AIndex : TPaperFormat);
    procedure ResizeImage;
    procedure ShowPos (x,y : integer);
    procedure SetToolbar (AtLeft : boolean);
    procedure ShowStatus;
    procedure InitPlotField;
    function GetBBox (AName         : string;
                      var BoxTyp    : TBoxTyp;
                      var Offs,Size : TFPoint) : boolean;
    procedure CopyPSFile (fsPS  : TWriteFileStream;
                          PSBox : TEPSBox);
    procedure PrintPSText (fsPS  : TWriteFileStream;
                           PSBox : TEPSBox);
    procedure SetViewPort;
    procedure MouseToCm (Pos         : TPoint;
                         var Xcm,Ycm : double);
    procedure CmToMouse (Xcm,Ycm : double;
                            var Pos : TPoint);
    procedure PlotBBox (const PSBox : TEPSBox;
                        Col         : integer);
    procedure ShowFrame (var FramePos : TRect;
                         PMode    : TPenMode);
    procedure LoadData (AName : string);
    procedure SaveData (AName : string);
  public
    { Public-Deklarationen }
  end;

var
  PrepsForm: TPrepsForm;

implementation

{$R *.DFM}

uses StrUtils, PlotTextDlg, InpDFloat, ShellApi, ShlObj, WinApiUtils, WinMath,
  ShellDirDlg, FileUtils, StringUtils, NumberUtils, InpText;

{ ------------------------------------------------------------------- }
constructor TEPSBox.Create (BoxSize : TFPoint);
begin
  PSBoxName:='';
  BoxTyp:=btDefault;
  Orig.X:=0; Orig.Y:=0; Offs.X:=0; Offs.Y:=0;
  Size.X:=BoxSize.X; Size.Y:=BoxSize.Y;
  with BFont do begin
    PSFontTyp:=DefFontTyp;
    PSSize:=DefFontSize;
    PSAngle:=0;
    PSStyle:=tsNormal;
    PSColor:=clBlack;
    end;
  end;

procedure TEPSBox.Assign (ABox : TEPSBox);
begin
  PSBoxName:=ABox.PSBoxName;
  BoxTyp:=ABox.BoxTyp;
  Orig:=ABox.Orig;
  Offs:=ABox.Offs;
  Size:=ABox.Size;
  BFont:=ABox.BFont;
  end;

{ ------------------------------------------------------------------- }
const
  IniExt = 'ini';
  CfgSekt = 'Config';
  BildSekt = 'Form';
  FileSekt = 'Files';
  GlobalSekt = 'Global';
  FontSekt = 'Font';
  PrtFileSekt = 'PrintFiles';

  IniBlatt = 'Name';
  IniSuffix = 'Suffix';
  IniPrtNr = 'Printer';
  IniPrtDir = 'PrintDir';
  IniPrtMode = 'PrintMode';
  IniFormat = 'Landscape';
  IniPSize  = 'PaperSize';
  IniHeight = 'PaperHeight';
  IniWidth = 'PaperWidth';
  IniFootOn = 'FootLine';
  IniFootLine = 'FootPosition';
  IniGridMode = 'Grid';
  IniRaster = 'GridWidth';
  IniFontType = 'FontType';
  IniFontSize = 'FontSize';
  IniFontStyle = 'FontStyle';
  IniFontColor = 'FontColor';
  IniToolbar = 'Toolbar';
  IniPsCmds  = 'IgnoreCommands';
  iniWorkDir = 'WorkDir';
  IniFilter  = 'Filter';

procedure TPrepsForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
  s        : string;
  err      : boolean;
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  PrtPfad:='';
  IniName:=Erweiter(GetDesktopFolder(CSIDL_APPDATA),PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    ArbPfad:=ReadString(CfgSekt,IniWorkDir,ArbPfad);
    Suffix:=ReadString(CfgSekt,IniSuffix,'');
    EPSFilter:=ReadString(CfgSekt,IniFilter,defFilter);
    psIgnore:=ReadString(CfgSekt,IniPsCmds,PsIgnore);
    TBLeftItem.Checked:=ReadBool(CfgSekt,IniToolbar,true);
    PrtNr:=ReadInteger(CfgSekt,IniPrtNr,DefLPT);
    if PrtNr=0 then begin
      PrintMode:=pmFile;
      PrtNr:=DefLPT;
      end;
    PrtFile:=defName; //ReadString(CfgSekt,IniPrtFile,);
    PrintMode:=TPrintMode(ReadInteger(CfgSekt,IniPrtMode,integer(PrintMode)));
    Querformat:=ReadBool(CfgSekt,IniFormat,true);
    PaperFormat:=TPaperFormat(ReadInteger(CfgSekt,IniPSize,integer(piA4)));
    with UserSize do begin
      Y:=ReadFloat (CfgSekt,IniHeight,PapA4.Y);
      X:=ReadFloat (CfgSekt,IniWidth,PapA4.X);
      end;
    FootNote:=ReadBool(CfgSekt,IniFootOn,false);
    s:=ReadString(CfgSekt,IniFootLine,'');
    err:=false;
    FootPosX:=ReadNxtDbl(s,'|',DefFootLeft,err);
    FootPosY:=ReadNxtDbl(s,'|',DefFootBottom,err);
    Usegrid:=ReadBool(CfgSekt,IniGridMode,true);
    with FormPos do begin
      Left:=ReadInteger (BildSekt,'Left',0);
      Top:=ReadInteger (BildSekt,'Top',0);
      Right:=ReadInteger (BildSekt,'Width',ClientWidth);
      Bottom:=ReadInteger (BildSekt,'Height',ClientHeight);
      end;
    WindowState:=TWindowState(ReadInteger (BildSekt,'State',Ord(wsNormal)));
    with TextFont do begin
      PSFontTyp:=ReadInteger(FontSekt,IniFontType,DefFontTyp);
      PSSize:=ReadInteger(FontSekt,IniFontSize,18);
      PSStyle:=ReadInteger(FontSekt,IniFontStyle,tsNormal);
      PSColor:=TColor(ReadInteger(FontSekt,IniFontColor,integer(clBlack)));
      end;
    Free;
    end;
  ProgVers:=GetFileVersionName(Application.Exename,Prog,Vers);
  Caption:=ProgVers+' [neu]';
  SetToolbar(TBLeftItem.Checked);
  FootBox:=TEPSBox.Create (DefSize);
  SetPaperSize (PaperFormat);
  InitScale(PapSize);  // Rnder fr Lineale
  PrintFootItem.Checked:=FootNote;
  BName:=''; Textline:='';
  (* Datei-Men erweitern *)
  FileList:=THistoryList.Create;
  with FileList do begin
    LoadFromIni (IniName,FileSekt);
    Menu:=DateiListe;
    OnAutoItemClick:=LoadDataListClick;
    end;
  SizeSet:=false; MNr:=-1; FrameOn:=false;
  FName:=''; BoxSize:=DefSize;
  ScaleFak:=DefScaleFak;
  GrPlot:=TPlot.Create (GrafikFeld.Canvas);
  EPSList:=TStringList.Create;
  chg:=false; TextCopy:=false; NewBox:=false;
  pn:=-1;
  PreviewBtn.Enabled:=false;
  end;

procedure TPrepsForm.FormDestroy(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfgSekt,IniWorkDir,ArbPfad);
    WriteString(CfgSekt,IniSuffix,Suffix);
    WriteString(CfgSekt,IniFilter,EPSFilter);
    WriteString(CfgSekt,IniPsCmds,psIgnore);
    WriteBool(CfgSekt,IniToolbar,TBLeftItem.Checked);
    WriteInteger(CfgSekt,IniPrtNr,PrtNr);
//    WriteString(CfgSekt,IniPrtFile,PrtFile);
    WriteInteger(CfgSekt,IniPrtMode,integer(PrintMode));
    WriteBool(CfgSekt,IniFormat,Querformat);
    WriteInteger(CfgSekt,IniPSize,integer(PaperFormat));
    with UserSize do begin
      WriteString (CfgSekt,IniHeight,FloatToStrF(Y,ffFixed,6,1));
      WriteString (CfgSekt,IniWidth,FloatToStrF(X,ffFixed,6,1));
      end;
    WriteBool(CfgSekt,IniFootOn,FootNote);
    WriteString(CfgSekt,IniFootLine,FloatToStrF(FootPosX,ffFixed,5,2)+
               '|'+FloatToStrF(FootPosY,ffFixed,5,2));
    WriteBool(CfgSekt,IniGridMode,Usegrid);
//    WriteInteger(CfgSekt,IniRaster,Raster);
    WriteInteger (BildSekt,'Left',Left);
    WriteInteger (BildSekt,'Top',Top);
    WriteInteger (BildSekt,'Width',ClientWidth);
    WriteInteger (BildSekt,'Height',ClientHeight);
    WriteInteger (BildSekt,'State',ord(WindowState));
    with TextFont do begin
      WriteInteger(FontSekt,IniFontType,PSFontTyp);
      WriteInteger(FontSekt,IniFontSize,PSSize);
      WriteInteger(FontSekt,IniFontStyle,PSStyle);
      WriteInteger(FontSekt,IniFontColor,integer(PSColor));
      end;
    Free;
    end;
  with FileList do begin
    SaveToIni (IniName,FileSekt,true);
    Free;
    end;
  EPSList.free; FootBox.Free;  
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if chg and ConfirmDialog(Prog,'Blatt speichern?') then SaveItemClick(Sender);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.FormShow(Sender: TObject);
begin
  PsPrintDialog.LoadFromIni(IniName,PrtFileSekt);
  end;

procedure TPrepsForm.FormActivate(Sender: TObject);
begin
  { Bildschirmfenster }
  Left:=FormPos.Left; Top:=FormPos.Top;
  ClientWidth:=FormPos.Right; ClientHeight:=FormPos.Bottom;
  RasterBtn.Down:=UseGrid;
  RasterOn.Checked:=UseGrid;
  if QuerFormat then begin
    QuerItem.Checked:=true; QuerBtn.Down:=true;
    end
  else begin
    HochItem.Checked:=true; HochBtn.Down:=true;
    end;
  case PrtNr of
  2 : Lpt21.Checked:=true;
  3 : Lpt31.Checked:=true;
  4 : Lpt41.Checked:=true;
    else Lpt11.Checked:=true;
    end;
  ShowStatus;
  InitPlotField;
  FormResize(Sender);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.SetToolbar (AtLeft : boolean);
var
  i : integer;
begin
  with ToolPanel do if AtLeft then begin
    Width:=34;
    Align:=alLeft;
    for i:=0 to ControlCount-1 do if (Controls[i] is TSpeedButton) then
        with (Controls[i] as TSpeedButton) do begin
      Left:=1; Top:=ToolButtonPos[Tag];
      end;
    end
  else begin
    Height:=34;
    Align:=alTop;
    for i:=0 to ControlCount-1 do if (Controls[i] is TSpeedButton) then
        with (Controls[i] as TSpeedButton) do begin
      Top:=1; Left:=ToolButtonPos[Tag];
      end;
    end;
  end;

procedure TPrepsForm.ShowStatus;
var
  s : string;
begin
  if length(PrtPfad)>0 then begin
    OutDirItem.Checked:=true;
    s:=PrtPfad;
    end
  else begin
    OutEpsDirItem.Checked:=true;
    s:=ArbPfad;
    end;
  with StatusBar.Panels[2] do
    if PrintMode<>pmPrinter then Text:=' Ausgabe nach '+Erweiter(s,PrtFile,'PS')
    else Text:=' Ausgabe nach LPT'+StrInt(PrtNr,0);
  StatusBar.Invalidate;
  end;

procedure TPrepsForm.StatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  with StatusBar.Canvas do begin
    with Panel do if Index=0 then begin
      Font.Style:=[fsBold];
      Font.Color:=clNavy;
      TextOut(Rect.Left+5,Rect.Top+2,'Format: '+PaperFormNames[PaperFormat]);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Zeichenfeld auf Papiergre einrichten *)
procedure TPrepsForm.InitPlotField;
var
  Field : TFPoint;
  Rand  : TFRect;
  i     : integer;
begin
  (* Zeichenflche *)
  with Field do if Querformat then begin
    X:=PapSize.Y+URand.Left+URand.Right;
    Y:=PapSize.X+URand.Bottom+URand.Top;
    end
  else begin
    X:=PapSize.X+URand.Left+URand.Right;
    Y:=PapSize.Y+URand.Bottom+URand.Top;
    end;
  with GrPlot do begin
    with Rand do begin
      Left:=0; Right:=0; Top:=0; Bottom:=0;
      end;
    with Field do XYRatio:=X/Y;
    SizeSet:=true;
    SetPlotField (Field,Rand);
    RawScale(Raster/10,0,MRaster,i);
    with URand do begin
      DefUserField (ufRel,Left,Right,Bottom,Top);
      RawScale(Left,Bottom,PRand.Left,PRand.Bottom);
      RawScale(Top,Right,PRand.Top,PRand.Right);
      end;
    end;
  end;

procedure TPrepsForm.SetPaperSize (AIndex : TPaperFormat);
begin
  PaperFormat:=AIndex;
  case AIndex of
  piA4 : begin // A4
         PapSize:=PapA4;
         A4FormItem.Checked:=true;
         end;
  piA3 : begin // A3
         PapSize:=PapA3;
         A3FormItem.Checked:=true;
         end;
  piA2 : begin // A2
         PapSize:=PapA2;
         A2FormItem.Checked:=true;
         end;
  piA1 : begin // A1
         PapSize:=PapA1;
         A1FormItem.Checked:=true;
         end;
  piA0 : begin // A0
         PapSize:=PapA0;
         A0FormItem.Checked:=true;
         end;
    else begin
      PapSize:=UserSize;
      IndiItem.Checked:=true;
      end;
    end;
  StatusBar.Invalidate;
  end;

procedure TPrepsForm.InitScale(ASize : TFPoint);
var
  x,dec : double;
begin
  x:=ASize.y/40;
  with URand do begin
    Left:=x; Right:=x/3;
    Bottom:=x; Top:=x/3;
    end;
  with Lineal do begin
    Schrift:=x/2; Abstand:=x/3; Grob:=x/5; Fein:=x/10;
    x:=ASize.y;
    dec:=pwr(10.0,aint(lg(abs(x))-0.05));
    x:=x/dec;
    if x<=2.1 then Skala:=0.1
    else if x<=5.1 then Skala:=0.2
    else Skala:=0.5;
    Skala:=Skala*dec;
    Raster:=round(Skala);    // Raster in mm = Skala in cm
//    FootPos:=Skala/5;
    end;
  with FootBox do begin
    Orig.x:=FootPosX; Orig.y:=FootPosY;
    with BFont do begin
      PSFontTyp:=1; PSSize:=round(Lineal.Schrift*CmToPt);
      PSStyle:=tsNormal; PSColor:=clBlack;
      end;
    PSBoxName:='[neu]';
    BoxTyp:=btDefault;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.InfoClick(Sender: TObject);
begin
  MessageDlg(ProgVers+#13+CopRgt,mtInformation,[mbOK],0);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.EpsPfadItemClick(Sender: TObject);
begin
  if ShellDirDialog.Execute ('Verzeichnis mit EPS-Dateien',false,true,false,'',ArbPfad) then ShowStatus;
  end;

procedure TPrepsForm.OutEpsDirItemClick(Sender: TObject);
begin
  PrtPfad:='';
  ShowStatus;
  end;

procedure TPrepsForm.OutDirItemClick(Sender: TObject);
var
  s : string;
begin
  if length(PrtPfad)=0 then s:=ArbPfad else s:=PrtPfad;
  if ShellDirDialog.Execute ('Verzeichnis mit EPS-Dateien',false,true,false,'',s) then begin
    PrtPfad:=s;
    Chg:=true;
    ShowStatus;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.EndBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.FilterItemClick(Sender: TObject);
begin
  FileFilterDialog.Execute ('Dateitypen',EpsFilter);
  end;

procedure TPrepsForm.SuffixItemClick(Sender: TObject);
begin
  if InputText(CursorPos,'Suffix fr Postscript-Datei','Dateinamen-Suffix:',false,Suffix) then begin
    PrtFile:=InsertNameSuffix(NewExt(ExtractFileName(BName),PExt),Suffix);
    ShowStatus;
    end;
  end;

{-----------------------------------------------------------------------}
(* Gre von GrafikFeld anpassen *)
procedure TPrepsForm.FormResize(Sender: TObject);
var
  f : double;
begin
  if SizeSet then with GrafikFeld do begin
    if Height<50 then ClientHeight:=50;
    if Width<50 then ClientWidth:=50;
    with GrSize do begin
      X:=Width; Y:=Height;
      f:=X/Y;
      with GScale do if XYRatio>f then begin
        X:=1; Y:=f/XYRatio;
        end
      else begin
        X:=XYRatio/f; Y:=1;
        end;
      end;
    end;
  end;

procedure TPrepsForm.LargerItemClick(Sender: TObject);
begin
  if ScaleFak>1 then begin
    ScaleFak:=ScaleFak/sqrt2;
    GrafikFeld.Invalidate;
    end;
  end;

procedure TPrepsForm.SmallerItemClick(Sender: TObject);
begin
  if ScaleFak<4096 then begin
    ScaleFak:=sqrt2*ScaleFak;
    GrafikFeld.Invalidate;
    end;
  end;

procedure TPrepsForm.FitItemClick(Sender: TObject);
begin
  ScaleFak:=DefScaleFak;
  GrafikFeld.Invalidate;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.SetViewPort;
var
  Zoom      : TFPoint;
begin
  with Zoom do begin
    X:=GScale.X*DefScaleFak/ScaleFak;
    Y:=GScale.Y*DefScaleFak/ScaleFak;
    HorzScrollbar.Visible:=X>1;
    VertScrollbar.Visible:=Y>1;
    end;
  with VertScrollBar do if Visible then SetFocus;
  with GrSize do begin
    X:=GrafikFeld.Width; Y:=GrafikFeld.Height;
    end;
  GrPlot.SetScreenWindowEx (GrSize,
            Point(HorzScrollbar.Position,VertScrollbar.Position),Zoom);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.LoadData (AName : string);
var
  DatFile  : TIniFile;
  i,n      : integer;
  s,fn     : string;
  Box      : TEPSBox;
  err      : boolean;
begin
  DatFile:=TIniFile.Create(AName);
  BName:=AName;
  PrtFile:=InsertNameSuffix(NewExt(ExtractFileName(BName),PExt),Suffix);
  with DatFile do begin
    with EPSList do begin
      Clear;
      n:=ReadInteger (GlobalSekt,'Count',0);
      Querformat:=ReadBool(GlobalSekt,IniFormat,false);
      if QuerFormat then begin
        QuerItem.Checked:=true; QuerBtn.Down:=true;
        end
      else begin
        HochItem.Checked:=true; HochBtn.Down:=true;
        end;
      PaperFormat:=TPaperFormat(ReadInteger(GlobalSekt,IniPSize,integer(piUser)));
      with UserSize do begin
        Y:=ReadFloat (GlobalSekt,IniHeight,PapSize.Y);
        X:=ReadFloat (GlobalSekt,IniWidth,PapSize.X);
        end;
      SetPaperSize(PaperFormat);
      InitScale(PapSize);
      PrintFootItem.Checked:=ReadBool(GlobalSekt,IniFootOn,FootNote);
      s:=ReadString(GlobalSekt,IniFootLine,'');
      PrtPfad:=ReadString(GlobalSekt,IniPrtDir,'');
      err:=false;
      with FootBox do begin
        Orig.X:=ReadNxtDbl(s,'|',FootPosX,err);
        Orig.Y:=ReadNxtDbl(s,'|',FootPosY,err);
        end;
      Raster:=ReadInteger(GlobalSekt,IniRaster,Raster);
      for i:=1 to n do begin
        s:='Frame'+ZStrInt(i,2);
        fn:=ReadString (s,'Name','');
        if length(fn)>0 then begin
          Box:=TEPSBox.Create(BoxSize);
          with Box do begin
            PSBoxName:=fn;
            BoxTyp:=TBoxTyp(ReadInteger (s,'BBox',integer(BoxTyp)));
            with Orig do begin
              X:=ReadFloat (s,'XOrigin',X);
              Y:=ReadFloat (s,'YOrigin',Y);
              end;
            with Offs do begin
              X:=ReadFloat (s,'XOffset',X);
              Y:=ReadFloat (s,'YOffset',Y);
              end;
            with Size do begin
              X:=ReadFloat (s,'XSize',X);
              Y:=ReadFloat (s,'YSize',Y);
              end;
            if (BoxTyp=btText) or (BoxTyp=btDefault) then with BFont do begin
              PSFontTyp:=ReadInteger (s,IniFontType,TextFont.PsFontTyp);
              PSSize:=ReadInteger (s,IniFontSize,TextFont.PSSize);
              PSStyle:=ReadInteger (s,IniFontStyle,TextFont.PSStyle);
              PSColor:=TColor(ReadInteger (s,IniFontColor,integer(TextFont.PsColor)));
              with GrPlot do begin
                SetTextHeight (PSSize/CmToPt);
                SetTextStyle (PSStyle);
                with Size do begin
                  x:=GetTextWidth(PSBoxName); y:=GetTextHeight;
                  end;
                end;
              end;
            end;
          AddObject (fn,Box);
          end;
        end;
      end;
    MNr:=-1;
    Free;
    end;
  PreviewBtn.Enabled:=false;
  ShowStatus;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.SaveData (AName : string);
var
  DatFile  : TIniFile;
  i        : integer;
  s        : string;
begin
  DatFile:=TIniFile.Create(AName);
  BName:=AName; chg:=false;
  PrtFile:=InsertNameSuffix(NewExt(ExtractFileName(BName),PExt),Suffix);
  with DatFile do begin
    with EPSList do if Count>0 then begin
      WriteInteger (GlobalSekt,'Count',Count);
      WriteBool(GlobalSekt,IniFormat,Querformat);
      WriteInteger(GlobalSekt,IniPSize,integer(PaperFormat));
      with UserSize do begin
        WriteString (GlobalSekt,IniHeight,FloatToStrF(Y,ffFixed,6,1));
        WriteString (GlobalSekt,IniWidth,FloatToStrF(X,ffFixed,6,1));
        end;
      WriteBool(GlobalSekt,IniFootOn,PrintFootItem.Checked);
      with FootBox do
        WriteString(GlobalSekt,IniFootLine,FloatToStrF(Orig.X,ffFixed,5,2)+
               '|'+FloatToStrF(Orig.Y,ffFixed,5,2));
      WriteInteger(GlobalSekt,IniRaster,Raster);
      WriteString(GlobalSekt,IniPrtDir,PrtPfad);
      for i:=0 to Count-1 do with (Objects[i] as TEPSBox) do begin
        s:='Frame'+ZStrInt(i+1,2);
        EraseSection (s);
        WriteString (s,'Name',PSBoxName);
        WriteInteger (s,'BBox',integer(BoxTyp));
        with Orig do begin
          WriteString (s,'XOrigin',FloatToStrF(X,ffFixed,6,1));
          WriteString (s,'YOrigin',FloatToStrF(Y,ffFixed,6,1));
          end;
        with Offs do begin
          WriteString (s,'XOffset',FloatToStrF(X,ffFixed,6,1));
          WriteString (s,'YOffset',FloatToStrF(Y,ffFixed,6,1));
          end;
        with Size do begin
          WriteString (s,'XSize',FloatToStrF(X,ffFixed,6,1));
          WriteString (s,'YSize',FloatToStrF(Y,ffFixed,6,1));
          end;
        if (BoxTyp=btText) or (BoxTyp=btText) then with BFont do begin
          WriteInteger (s,IniFontType,PSFontTyp);
          WriteInteger (s,IniFontSize,PSSize);
          WriteInteger (s,IniFontStyle,PSStyle);
          WriteInteger (s,IniFontColor,integer(PSColor));
          end;
        end;
      end;
    Free;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.ClearBtnClick(Sender: TObject);
begin
  if chg and ConfirmDialog (Prog,'Blatt speichern?') then SaveItemClick(Sender);
  if ConfirmDialog (Prog,'Alles lschen?') then begin
    FName:='';
    EPSList.Clear;
    BName:='';
    PrtFile:=defName;
    Caption:=ProgVers+' [neu]';
    Chg:=false; TextCopy:=false;
    GrafikFeld.Invalidate;
    ShowStatus;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.LoadBtnClick(Sender: TObject);
begin
  if chg and ConfirmDialog (Prog,'Blatt speichern?') then SaveItemClick(Sender);
  with OpenDialog do begin
    if length(BName)>0 then InitialDir:=ExtractFilePath(BName)
    else InitialDir:=ArbPfad;
    DefaultExt:=BExt;
    Filename:=''; Filter:='Blattdaten|*.'+BExt+'|alle|*.*';
    Title:='Lade Blatt';
    if Execute then begin
      FileList.AddString(FileName);
      Caption:=ProgVers+' ['+ExtractFilename(Filename)+']';
      LoadData(Filename);
      FootBox.PSBoxName:=StripPath(Filename,60);
      InitPlotField;
      FormResize(Sender);
      GrafikFeld.Invalidate;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.SaveItemClick(Sender: TObject);
begin
  if length(BName)>0 then SaveData (BName)
  else SaveAsBtnClick(Sender);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.SaveAsBtnClick(Sender: TObject);
begin
  with SaveDialog do begin
    InitialDir:=ArbPfad;
    Filename:=DelExt(ExtractFilename(FName));
    DefaultExt:=BExt;
    Filter:='Blattdaten|*.'+BExt;
    Title:='Lade Blatt';
    if Execute then begin
      FileList.AddString(FileName);
      SaveData (Filename);
      FootBox.PSBoxName:=StripPath(Filename,60);
      Caption:=ProgVers+' ['+ExtractFilename(Filename)+']';
      GrafikFeld.Invalidate;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Klick auf Dateilistenmen *)
procedure TPrepsForm.LoadDataListClick (Sender     : TObject;
                                        FileName   : string);
begin
  if FileExists (FileName) then begin
    FileList.AddString(FileName);
    BName:=FileName;
    Caption:=ProgVers+' ['+ExtractFilename(Filename)+']';
    LoadData(BName);
    FootBox.PSBoxName:=StripPath(Filename,60);
    InitPlotField;
    FormResize(Sender);
    GrafikFeld.Invalidate;
    end
  else begin
    ShowMessage('Datei:' + FileName + ' nicht gefunden!');
    FileList.RemString (FileName);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Bounding-Box aus PS-Datei lesen *)
function TPrepsForm.GetBBox (AName         : string;
                             var BoxTyp    : TBoxTyp;
                             var Offs,Size : TFPoint) : boolean;
var
  fsEps  : TReadFileStream;
  zeile  : AnsiString;
  i      : integer;
  x1,x2,
  y1,y2  : word;

  (* Integerzahl aus String bis Space *)
  function Cvt (var zeile : AnsiString;
                def       : integer) : integer;
  var
    i,ic,n : integer;
  begin
    while (length(zeile)>0) and (zeile[1]=#32) do delete (zeile,1,1);
    if length(zeile)>0 then begin
      i:=pos(#32,zeile); if i=0 then i:=succ(length(zeile));
      val(copy(zeile,1,pred(i)),n,ic); delete(zeile,1,i);
      if ic>0 then Cvt:=Def else Cvt:=n;
      end
    else Cvt:=Def;
    end;

begin
  Result:=false;
  try
    fsEps:=TReadFileStream.Create(AName,fmShareDenyNone);
  except
    ErrorDialog('EPS-Datei: '#13+StripPath(AName,40)+#13' nicht gefunden!');
    Exit;
    end;
  Zeile:=fsEps.ReadAnsiString;
  i:=1;
  try
    if copy(zeile,1,2)='%!' then begin
      repeat
        Zeile:=fsEps.ReadAnsiString;
        inc(i);
        until (i>100) or (copy(zeile,1,14)='%%BoundingBox:');
      if i<=100 then begin (* BBox gefunden *)
        BoxTyp:=btBoundingBox;
        delete(zeile,1,14);
        x1:=cvt (zeile,0); y1:=cvt (zeile,0);
        with Size do begin
          x2:=cvt (zeile,round(x*CmToPt));
          y2:=cvt (zeile,round(y*CmToPt));
          end;
        with Offs do begin
          x:=x1/CmToPt; y:=y1/CmToPt;
          end;
        with Size do begin
          x:=(x2-x1)/CmToPt; y:=(y2-y1)/CmToPt;
          end;
        end
      else ErrorDialog('Keine Bounding Box in '#13+StripPath(AName,40)+
                 #13'Verwende Standardgre');
      Result:=true;
      end
    else ErrorDialog('Kein Postscript in '#13+StripPath(AName,40));
  finally
    fsEps.Free;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.InsertBtnClick(Sender: TObject);
var
  Box   : TEPSBox;
  x0,y0 : double;
  Pos   : TPoint;
  i     : integer;
  s,t   : string;
begin
  with OpenDialog do begin
    if length(FName)>0 then InitialDir:=ExtractFilePath(FName)
    else InitialDir:=ArbPfad;
    DefaultExt:='eps';
    Filename:='';
    s:=EpsFilter; i:=0;
    if length(Fname)>0 then repeat
      ReadNxtStr(s,'|');
      t:=ReadNxtStr(s,'|');  // Extensions
      if AnsiContainsText (t,'*.'+GetExt(FName)) then begin
        i:=1;
        while (i<=length(t)) and (PosEx(';',t,i)>0) do inc(i);
        if i<2 then Filename:=''
        else Filename:=DelExt(ExtractFilename(Fname))+'.*';
        end;
      until (length(s)=0) or (i>0);
    Filter:=EpsFilter;
    Title:='EPS-Datei auswhlen';
    if Execute then begin
      FName:=Filename;
      if length(BName)=0 then PrtFile:=InsertNameSuffix(NewExt(ExtractFileName(FName),PExt),Suffix);
      Box:=TEPSBox.Create(BoxSize);
      with Box do if GetBBox (FName,BoxTyp,Offs,Size) then begin
        PSBoxName:=FName;
        i:=EPSList.AddObject (FName,Box);
        with GrPlot do begin
          with PlotField do begin
            x0:=(Right-Left)/4;
            y0:=(Top-Bottom)/4;
            end;
          with Box do begin
            Scale(x0,y0,FramePos.Left,FramePos.Bottom);
            Scale(Size.X,Size.Y,FramePos.Right,FramePos.Top);
            with FramePos do
              CmToMouse(x0,y0,Pos);
              SetCursorPos(Pos.X+GrafikFeld.ClientOrigin.X,
                           Pos.Y+GrafikFeld.ClientOrigin.Y);
            end;
          end;
        MNr:=i; NewBox:=true;
        GrafikFeld.Invalidate;
        chg:=true;
        end;
      ShowStatus;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.UpdateBtnClick(Sender: TObject);
var
  i : integer;
begin
  with EPSList do if Count>0 then begin
    for i:=Count-1 downto 0 do with (Objects[i] as TEPSBox) do if BoxTyp=btBoundingBox then begin
      if not GetBBox (PSBoxName,BoxTyp,OldOrg,Size) then Delete (i);
      end;
    end;
  GrafikFeld.Invalidate;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.InsertTextItemClick(Sender: TObject);
var
  Box   : TEPSBox;
  x0,y0 : double;
  Pos   : TPoint;
begin
  if GetPlotText(CursorPos,'Text einfgen','Text:',true,
                 FontList[TextFont.PSFontTyp].WinName,Textline,false) then begin
    Box:=TEPSBox.Create(BoxSize);
    with Box do begin
      with BFont do begin
        PSFontTyp:=TextFont.PSFontTyp;
        PSSize:=TextFont.PSSize;
        PSStyle:=TextFont.PSStyle;
        PSColor:=TextFont.PSColor;
        end;
      PSBoxName:=Textline;
      BoxTyp:=btText;
      with GrPlot do begin
        SetTextFont(FontList[BFont.PSFontTyp].WinName);
        SetTextHeight (BFont.PSSize/CmToPt);
        SetTextStyle (BFont.PSStyle);
        with Size do begin
          x:=GetTextWidth(Textline); y:=GetTextHeight;
          end;
        with PlotField do begin
          x0:=(Right-Left)/4;
          y0:=(Top-Bottom)/4;
          end;
        RawScale(x0,y0,FramePos.Left,FramePos.Bottom);
        RawScale(Size.X,Size.Y,FramePos.Right,FramePos.Top);
        CmToMouse(x0,y0,Pos);
        SetCursorPos(Pos.X+GrafikFeld.ClientOrigin.X,
                     Pos.Y+GrafikFeld.ClientOrigin.Y);
        end;
      MNr:=EPSList.AddObject (FName,Box);
      GrafikFeld.Invalidate;
      chg:=true;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Box zeichnen*)
procedure TPrepsForm.PlotBBox (const PSBox : TEPSBox;
                               Col         : integer);
var
  s             : string;
  Frame         : TFRect;
  OldPen        : TPen;
  Color         : TColor;
begin
  with GrPlot do begin
    with PSBox do begin
      with PlotCanvas do begin
        OldPen:=Pen;
        if BoxTyp=btBoundingBox then begin
          Col:=abs(Col);
          with Pen do begin
            Color:=StdPlotColor[Col]; Style:=psDash; Mode:=pmCopy;
            end;
          with Frame do begin
            Left:=UserScale(1,Orig.X);
            Right:=UserScale(1,Orig.X+Size.X);
            Bottom:=UserScale(5,Orig.Y);
            Top:=UserScale(5,Orig.Y+Size.Y);
            CPlot(Left,Top,pmUp,mmNoMark); CPlot(Right,Top,pmDown,mmNoMark);
            CPlot(Right,Bottom,pmDown,mmNoMark); Cplot(Left,Bottom,pmDown,mmNoMark);
            CPlot(Left,Top,pmDown,mmNoMark);
            Pen:=OldPen;
            end;
          with Lineal do begin  // Schriftskalierung
            SetTextHeight(Schrift);
            SetTextFont(DefFontName); SetTextStyle(tsNormal);
            PlotStr (UserScale(1,Orig.X+Grob),UserScale(5,Orig.Y+Size.Y-Fein),
                  StripPath(PSBoxName,20),false,alLeftTop,Col);
            if BoxTyp=btBoundingBox then s:='BBox:' else s:='Std.:';
            with Size do
              PlotStr (UserScale(1,Orig.X+Grob),UserScale(5,Orig.Y+Fein),
                  s+FloatToStrF(x,ffFixed,5,1)+'x'+
                  FloatToStrF(y,ffFixed,5,1),false,alLeftBottom,Col);
            end;
          end
        else with BFont do begin
          SetTextHeight (PSSize/CmToPt);
          SetTextStyle (PSStyle);
          SetTextDir (PSAngle);
          if col<0 then color:=PSColor else color:=StdPlotColor[Col];
          PlotFontStr (UserScale(1,Orig.X),UserScale(5,Orig.Y),PSBoxName,BoxTyp=btText,
                       alLeftBottom,color,FontList[PSFontTyp].WinName);
          end;
        end;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.GrafikFeldPaint(Sender: TObject);
var
  BRaster,i,
  ix,x,y      : integer;
  Pt          : TPoint;
begin
  SetViewPort;
  {Grafik zeichnen }
  with GrPlot do begin
    InitPlot(true,clBtnFace);                { Zeichenfeld lschen, Farbe wei }
    (* Bereich fr Lineale *)
    with Userfield do PlotRect (Left,Bottom,Right,Top,ClWhite);
    (* Benutzerfeld definieren *)
    with Userfield do begin
      CPlot(Left,Bottom,pmUpOrigin,mmNoMark);    { Nullpunkt festlegen }
      DefUserScale (1,stHorLin,0.0,Right-Left);
      DefUserScale (5,stVertLin,0.0,Top-Bottom);
      end;
    (* Lineale *)
//    SetTextHeight(0.3);
//    SetAxisProp (0,0.3,0.2,0.15,0.075);
    with Lineal do begin
      SetAxisProp (0,Schrift,0.02,Abstand,Grob,Fein);
      Axis (1,0,Raster,10,[asBottom],black);
      Axis (5,0,Raster,10,[asLeft],black);
      end;
    BackColor:=clWhite; SetTransparent(true);
    (* Rasterpunkte *)
    BRaster:=MRaster;
    Pt.X:=0; Pt.Y:=0;
    WindowToView (Pt); ix:=Pt.x;
    Pt.X:=BRaster; Pt.Y:=0;
    WindowToView (Pt); ix:=abs(ix-Pt.x);
    if ix<2 then BRaster:=BRaster*10
    else if ix<5 then BRaster:=BRaster*5
    else if ix<10 then BRaster:=BRaster*2;
    with WindField do begin
      x:=Left+PRand.Left+BRaster;
      while x<Right-PRand.Right do begin
        y:=Bottom+PRand.Bottom+BRaster;
        while y<Top-PRand.Top do begin
          PlotCanvas.Pixels[x,y]:=clBlack;
          inc(y,BRaster);
          end;
        inc(x,BRaster);
        end;
      end;
    (* Bounding-Boxen zeichnen *)
    with EPSList do if Count>0 then begin
      for i:=0 to Count-1 do if i<>MNr then begin
        if (pn<0) or (pn<>i) then PlotBBox (Objects[i] as TEPSBox,-red)
        else PlotBBox (Objects[i] as TEPSBox,-green);
        end;
      end;
    end;
  if PrintFootItem.Checked then PlotBBox (FootBox,clBlack);
  if MNr>=0 then begin
    Screen.Cursor:=crSizeAll;
    ShowFrame (FramePos,pmXor);
    end
  else Screen.Cursor:=crdefault;
  end;

{ ------------------------------------------------------------------- }
(* Maus-Position in Grafikfeld in Koord. in cm umrechnen *)
procedure TPrepsForm.MouseToCm (Pos         : TPoint;
                                var Xcm,Ycm : double);
begin
  with GrPlot do begin
    SetViewPort;
    ViewToWindow (Pos);
    InvCScale (Pos.X,Pos.y,xcm,ycm);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Koord. in cm in Maus-Position in Grafikfeld umrechnen *)
procedure TPrepsForm.CmToMouse (Xcm,Ycm : double;
                            var Pos : TPoint);
begin
  with GrPlot do begin
    CScale (xcm,ycm,Pos.X,Pos.y);
    SetViewPort;
    WindowToView (Pos);
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.ShowFrame (var FramePos : TRect;
                                PMode        : TPenMode);
var
  Box : TRect;
  x   : integer;
begin
  with GrPlot do begin
    with FramePos do begin
      x:=WindField.Left+PRand.Left;
      if (Left<x) then Left:=x;
      x:=WindField.Right-PRand.Right;
      if (Left+Right>x) then Left:=x-Right;
      Box.Left:=Left; Box.Right:=Left+Right;
      x:=WindField.Bottom+PRand.Bottom;
      if (Bottom<x) then Bottom:=x;
      x:=WindField.Top-PRand.Top;
      if (Bottom+Top>x) then Bottom:=x-Top;
      Box.Bottom:=Bottom; Box.Top:=Bottom+Top;
      end;
    PlotFrame (Box,FrameCol,psSolid,PMode);
    end;
  end;

procedure TPrepsForm.ShowPos (x,y : integer);
var
  x0,y0 : double;
begin
  with GrPlot do begin
    InvCScale (x,y,x0,y0);
    with UserField do begin
      if x0<Left then x0:=Left;
      if x0>Right then x0:=Right;
      if y0<Bottom then y0:=Bottom;
      if y0>Top then y0:=Top;
      end;
    end;
  StatusBar.Panels[1].Text:=' '+FloatToStrF(x0,ffFixed,4,1)+' cm / '+
                                FloatToStrF(y0,ffFixed,4,1)+' cm';
  end;

procedure TPrepsForm.GrafikFeldMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  MPos.X:=X; MPos.Y:=Y;
  SetViewPort;
  GrPlot.ViewToWindow (MPos);
  if UseGrid then with MPos do begin
    X:=((X-PRand.Left) div MRaster)*MRaster+PRand.Left;
    Y:=((Y-PRand.Bottom) div MRaster)*MRaster+PRand.Bottom;
    end;
  if MNr>=0 then begin
    ShowFrame (FramePos,pmXor);
    with FramePos do begin
      Left:=MPos.X; Bottom:=MPos.Y;
      end;
    ShowFrame (FramePos,pmXor);
    with FramePos do ShowPos (Left,Bottom);
    FrameOn:=true;
    end
  else with MPos do ShowPos (x,y);
  end;

procedure TPrepsForm.GrafikFeldMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i     : integer;
  found : boolean;
  xc,yc : double;
  Pos   : TPoint;
begin
  if MNr>=0 then with (EPSList.Objects[MNr] as TEPSBox) do begin
    if Button=mbLeft then begin
      with FramePos do GrPlot.InvCScale (Left,Bottom,Orig.X,Orig.Y);
      end
    else begin
      if TextCopy or NewBox then begin
        EPSList.Delete(MNr);
        TextCopy:=false;
        end
      else Orig:=OldOrg;
      end;
    MNr:=-1; chg:=true; NewBox:=false;
    ShowFrame (FramePos,pmXor);
    GrafikFeld.Invalidate;
    end
  else begin
    Pos.X:=X; Pos.Y:=Y;
    MouseToCm (Pos,xc,yc);
    i:=0; found:=false;
    with WPos do begin
      x:=Pos.X+GrafikFeld.ClientOrigin.X;
      y:=Pos.Y+GrafikFeld.ClientOrigin.Y;
      end;
    with EPSList do begin
      while (i<Count) and not found do
          with Objects[i] as TEPSBox do begin
        found:=(Orig.x<=xc) and (xc<=Orig.x+Size.x) and
               (Orig.y<=yc) and (yc<=Orig.y+Size.y);
        if not found then inc(i);
        end;
      with GrPlot do if found then with Objects[i] as TEPSBox do begin
        xc:=Orig.x; yc:=Orig.y;
        RawScale(xc,yc,FramePos.Left,FramePos.Bottom);
        RawScale(Size.X,Size.Y,FramePos.Right,FramePos.Top);
        if Button=mbLeft then begin
          CmToMouse(xc,yc,Pos);
          SetCursorPos(Pos.X+GrafikFeld.ClientOrigin.X,
                         Pos.Y+GrafikFeld.ClientOrigin.Y);
          MNr:=i;
          OldOrg:=Orig;
          end
        else if Button=mbRight then begin
          PlotBBox (Objects[i] as TEPSBox,green);
          pn:=i;
          if (Objects[i] as TEPSBox).BoxTyp=btText then begin
            with WPos do TextPopup.Popup (x,y);
            end
          else begin
            with WPos do EPSPopup.Popup (x,y);
            end;
          end;
        end;
      GrafikFeld.Invalidate;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.EpsDelItemClick(Sender: TObject);
begin
  if (pn>=0) and ConfirmDialog(WPos,'Box lschen?') then begin
    EPSList.delete(pn);
    GrafikFeld.Invalidate;
    chg:=true;
    end;
  pn:=-1;
  end;

procedure TPrepsForm.EpsUpdateItemClick(Sender: TObject);
begin
  if pn>=0 then with EPSList,(Objects[pn] as TEPSBox) do begin
    if not GetBBox (PSBoxName,BoxTyp,OldOrg,Size) then Delete (pn);
    GrafikFeld.Invalidate;
    end;
  pn:=-1;
  end;

procedure TPrepsForm.EpsReplItemClick(Sender: TObject);
begin
  if pn>=0 then with EPSList,(Objects[pn] as TEPSBox) do begin
    with OpenDialog do begin
      InitialDir:=ExtractFilePath(PSBoxName);
      DefaultExt:='eps';
      Filename:=DelExt(ExtractFilename(PSBoxName))+'.*';
      Filter:=EpsFilter;
      Title:='EPS-Datei auswhlen';
      if Execute then begin
        PSBoxName:=Filename;
        GetBBox (PSBoxName,BoxTyp,OldOrg,Size);
        GrafikFeld.Invalidate;
        chg:=true;
        end;
      end;
    end;
  pn:=-1;
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.DelTextItemClick(Sender: TObject);
begin
  if (pn>=0) and  ConfirmDialog(WPos,'Text lschen?') then begin
    EPSList.delete(pn);
    GrafikFeld.Invalidate;
    chg:=true;
    end;
  pn:=-1;
  end;

procedure TPrepsForm.EditTextItemClick(Sender: TObject);
begin
  if pn>=0 then with (EPSList.Objects[pn] as TEPSBox) do begin
    if GetPlotText(CursorPos(Point(-20,-50)),'Text ndern','Text:',true,
                 FontList[BFont.PSFontTyp].WinName,PSBoxName,false) then begin
      with GrPlot,BFont do begin
        SetTextFont(FontList[PSFontTyp].WinName);
        SetTextHeight (PSSize/CmToPt);
        SetTextStyle (PSStyle);
        with Size do begin
          x:=GetTextWidth(PSBoxName); y:=GetTextHeight;
          end;
        end;
      GrafikFeld.Invalidate;
      chg:=true;
      end;
    end;
  pn:=-1;
  end;

procedure TPrepsForm.TextFontItemClick(Sender: TObject);
begin
  if pn>=0 then with (EPSList.Objects[pn] as TEPSBox) do begin
    if PSFontDialog.Execute(CursorPos,BFont) then begin
      with GrPlot,BFont do begin
        SetTextFont(FontList[PSFontTyp].WinName);
        SetTextHeight (PSSize/CmToPt);
        SetTextStyle (PSStyle);
        with Size do begin
          x:=GetTextWidth(PSBoxName); y:=GetTextHeight;
          end;
        end;
      GrafikFeld.Invalidate;
      chg:=true;
      end;
    end;  
  pn:=-1;
  end;

procedure TPrepsForm.CopyTextItemClick(Sender: TObject);
var
  Box   : TEPSBox;
begin
  if pn>=0 then with EPSList do begin
    Box:=TEPSBox.Create (DefSize);
    Box.Assign(Objects[pn] as TEPSBox);
    MNr:=AddObject(Strings[pn],Box);
    OldOrg:=Box.Orig;
    TextCopy:=true;
    end;
  pn:=-1;
  end;

{ ------------------------------------------------------------------- }
  // EPS-Header
const
  psBCm = '%!PS-Adobe-3.0 EPSF-3.0';
  psBbx = '%%%%BoundingBox: 0 0 %u %u';
  psTit = '%%%%Title: %s';
  psDte = '%%%%CreationDate: %s';
  psCre = '%%Creator: PrintEPS by J. Rathlev';
  psDat = '%%DocumentData: Clean7Bit';
  psLev = '%%%%LanguageLevel: %s';
  psPgs = '%%Pages: 1';
  psECm = '%%EndComments';
  psBPl = '%%BeginProlog';

  psEPl = '%%EndProlog';
  psPage = '%%Page: 0 1';

  psUnit = '/cm {28.35 mul} def';
  psCR = '/copypage {} bind def /showpage {} bind def /erasepage {} bind def /setpagedevice {} bind def';
  pstr = '%s cm 0 translate 90 rotate';
  pstl = '%s cm %s cm translate';
  pscp = '/clippath {newpath %s cm %s cm moveto %s cm %s cm lineto %s cm %s cm lineto closepath} def';
  ps7 = 'showpage end';
  psBD = '%%BeginDocument: ';
  psED = '%%EndDocument';

(* Postscript-Datei kopieren *)
procedure TPrepsForm.CopyPSFile (fsPS  : TWriteFileStream;
                                 PSBox : TEPSBox);
var
  s,t    : AnsiString;
  fsEps  : TReadFileStream;
begin
  try
    fsEps:=TReadFileStream.Create(PSBox.PSBoxName,fmShareDenyNone);
  except
    ErrorDialog('EPS-Datei: '#13+StripPath(PSBox.PSBoxName,40)+#13' nicht gefunden!');
    Exit;
    end;
  with PSBox do begin
    fsPS.WriteAnsiStringLine ('save');
    fsPS.WriteAnsiStringLine (psCR);       (* Showpage ausschalten *)
   (* Nullpunkt verschieben *)
    fsPS.WriteAnsiStringLine (Format(pstl,[FloatToStrX(Orig.x-Offs.x,3,1,Punkt),
                              FloatToStrX(Orig.y-Offs.y,3,1,Punkt)]));
    // ClipPath
    s:=Format(pscp,[FloatToStrX(Offs.x,3,1,Punkt),FloatToStrX(Offs.y,3,1,Punkt),
                    FloatToStrX(Offs.x+Size.x,3,1,Punkt),FloatToStrX(Offs.y,3,1,Punkt),
                    FloatToStrX(Offs.x+Size.x,3,1,Punkt),FloatToStrX(Offs.y+Size.y,3,1,Punkt)]);
    fsPS.WriteAnsiStringLine (s);
    if AnsiStartsText(ArbPfad,PSBox.PSBoxName) then
      s:=ExtractRelativePath(IncludeTrailingPathDelimiter(ArbPfad),PSBox.PSBoxName)
    else s:=PSBox.PSBoxName;
    fsPS.WriteAnsiStringLine (psBD+s);
    try
      while not fsEps.EOF do begin
        s:=fsEps.ReadAnsiString;
        if (length(s)>0) then begin
          t:=copy(s,1,2);
          fsPS.WriteAnsiStringLine (s);
          end;
        end;
    finally
      fsEps.Free;
      end;
    fsPS.WriteAnsiStringLine (psED);
    fsPS.WriteAnsiStringLine ('restore');
    end;
  end;

{ ------------------------------------------------------------------- }
(* Postscript-Text ausgeben *)
procedure TPrepsForm.PrintPSText (fsPS  : TWriteFileStream;
                                  PSBox : TEPSBox);
var
  s      : AnsiString;

  // Sonderzeichen ersetzen
  function ReplSpecChar (s : string) : string;
  var
    i : integer;
    t : string;
  begin
    t:='';
    for i:=1 to length(s) do begin
      case s[i] of
      '(' : t:=t+'\(';
      ')' : t:=t+'\)';
      '\' : t:=t+'\\';
      else t:=t+s[i];
        end;
      end;
    Result:=t;
    end;

begin
  with PSBox do begin
//    fsPS.WriteAnsiStringLine ('save');
    with BFont do begin
      fsPS.WriteAnsiStringLine ('StEnc enc');
      fsPS.WriteAnsiStringLine ('/'+FontList[PSFontTyp].PSName[PSStyle]+' '+IntToStr(PSSize)+' selfont');
//      fsPS.WriteAnsiStringLine ('/'+FontList[PSFontTyp].PSName[PSStyle]+' findfont');
//      fsPS.WriteAnsiStringLine (IntToStr(PSSize)+' scalefont setfont');
      end;
    with Orig do                    (* Anfangspunkt *)
      fsPS.WriteAnsiStringLine (FloatToStrX(x,4,1,Punkt)+' cm '+FloatToStrX(y,4,1,Punkt)+' cm moveto');
    s:=ReplSpecChar(PSBoxName);
    fsPS.WriteAnsiStringLine ('('+s+') show');
//    fsPS.WriteAnsiStringLine ('restore');
    end;
  end;

{ ------------------------------------------------------------------- }
(* Drucken *)
procedure TPrepsForm.PrintBtnClick(Sender: TObject);
var
  pf   : string;
  fsPS : TWriteFileStream;
  i    : integer;
  ok   : boolean;
begin
  pf:='LPT'+StrInt(PrtNr,0);
  ok:=PsPrintDialog.Execute(pf,PrtPfad,PrtFile,PrintMode);
  if ok then begin
    PreviewBtn.Enabled:=PrintMode>=pmFile;
    if PrintMode<>pmPrinter then begin
      ShowStatus;
      if OutDirItem.Checked then pf:=PrtPfad else pf:=ArbPfad;
      pf:=Erweiter(pf,PrtFile,PExt);
      PreviewBtn.Enabled:=true;
      end
    else PreviewBtn.Enabled:=false;
    try
      fsPS:=TWriteFileStream.Create(pf,0);
    except
      ErrorDialog('Fehler beim ffnen von '+pf);
      Exit;
      end;
    try
      with fsPS do begin
        (* PS-Header *)
        WriteAnsiStringLine (psBCm);
        WriteAnsiStringLine (Format(psTit,[ExtractFileName(pf)]));
        WriteAnsiStringLine (Format(psDte,[FormatDateTime('yyyy-mm-dd hh:nn',Now)]));
        WriteAnsiStringLine (psCre);
        WriteAnsiStringLine (psDat);
        WriteAnsiStringLine (Format(psLev,['2']));

        with PapSize do
          WriteAnsiStringLine (Format(psBbx,[round(x*CmToPt),round(y*CmToPt)]));
        WriteAnsiStringLine (psECm);
        WriteAnsiStringLine (psBPl);
        WriteAnsiStringLine ('/RpDict 200 dict def');
        WriteAnsiStringLine ('RpDict begin');
        WriteAnsiStringLine ('currentdict /pgsave save put');
        WriteAnsiStringLine ('/StEnc ISOLatin1Encoding 256 array copy def');
        WriteAnsiStringLine ('/Inifont {/rpf exch def findfont dup length dict /nf exch def'+sLineBreak+
                          '  {1 index /FID ne {nf 3 1 roll put} {pop pop} ifelse } forall'+sLineBreak+
                          '  nf /Encoding StEnc put rpf nf definefont pop} def');
        WriteAnsiStringLine ('/selfont {exch /RpFont Inifont /RpFont /rpf exch def'+sLineBreak+
                          '  rpf findfont exch scalefont setfont} def');
        WriteAnsiStringLine ('/enc {/StEnc exch def} def');
        WriteAnsiStringLine (psUnit);

        WriteAnsiStringLine (psEPl);      // End Prolog
        WriteAnsiStringLine (psPage);

        if QuerFormat then WriteAnsiStringLine (Format(pstr,[FloatToStrX(PapSize.x,4,1,Punkt)]));
        with EPSList do for i:=0 to Count-1 do begin
          WriteAnsiStringLine ('');
          with (Objects[i] as TEPSBox) do if BoxTyp=btText then begin
            PrintPSText (fsPS,Objects[i] as TEPSBox);
            end
          else CopyPsFile (fsPS,Objects[i] as TEPSBox);
          end;
        if PrintFootItem.Checked then PrintPSText (fsPS,FootBox);
        WriteAnsiStringLine (ps7);
        end;
    finally
      fsPS.Free;
      end;
    if PrintMode=pmView then PreviewBtnClick(Sender);
    end;
  end;

// Vorschau (erfordert Ghostview mit Verknpfung aus .ps)
procedure TPrepsForm.PreviewBtnClick(Sender: TObject);
var
  s : string;
begin
  if OutDirItem.Checked then s:=PrtPfad else s:=ArbPfad;
  ShellExecute(Handle,'open',pchar(Erweiter(s,PrtFile,PExt)),nil,nil,SW_SHOW);
  end;

{ ------------------------------------------------------------------- }
procedure TPrepsForm.LPT11Click(Sender: TObject);
begin
  PrtNr:=1;
  LPT11.Checked:=true;
  ShowStatus;
  end;

procedure TPrepsForm.LPT21Click(Sender: TObject);
begin
  PrtNr:=2;
  LPT21.Checked:=true;
  ShowStatus;
  end;

procedure TPrepsForm.LPT31Click(Sender: TObject);
begin
  PrtNr:=3;
  LPT31.Checked:=true;
  ShowStatus;
  end;

procedure TPrepsForm.LPT41Click(Sender: TObject);
begin
  PrtNr:=4;
  LPT41.Checked:=true;
  ShowStatus;
  end;

procedure TPrepsForm.QuerItemClick(Sender: TObject);
begin
  Querformat:=true; QuerItem.Checked:=true;
  ResizeImage;
  end;

procedure TPrepsForm.HochItemClick(Sender: TObject);
begin
  Querformat:=false; HochItem.Checked:=true;
  ResizeImage;
  end;

procedure TPrepsForm.ResizeImage;
begin
  if EPSList.Count>0 then chg:=true;
  InitPlotField;
  FormResize(self);
  GrafikFeld.Invalidate;
  end;

procedure TPrepsForm.RasterBtnClick(Sender: TObject);
begin
  UseGrid:=RasterBtn.Down;
  RasterOn.Checked:=UseGrid;
  end;

procedure TPrepsForm.RasterOnClick(Sender: TObject);
begin
  UseGrid:=not UseGrid;
  RasterBtn.Down:=UseGrid;
  RasterOn.Checked:=UseGrid;
  end;

procedure TPrepsForm.BoxsizeItemClick(Sender: TObject);
begin
  SizeDialog.Execute (ProgVers,'Boxgre',BoxSize);
  end;

procedure TPrepsForm.RasterItemClick(Sender: TObject);
var
  i : integer;
begin
  if NumDialog (ProgVers,'Raster in mm:',1,25,1,imFixed,Raster) then begin
    GrPlot.RawScale(Raster/10,0,MRaster,i);
    GrafikFeld.Invalidate;
    end;
  end;

procedure TPrepsForm.PSFontItemClick(Sender: TObject);
begin
  PSFontDialog.Execute(CursorPos,TextFont)
  end;

procedure TPrepsForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (MNr>=0) then with (EPSList.Objects[MNr] as TEPSBox) do begin
    if (Key=VK_RETURN) then begin
      GrPlot.InvCScale (MPos.X,MPos.y,Orig.X,Orig.Y);
      Orig.X:=Orig.X-Size.X/2; Orig.Y:=Orig.Y-Size.Y/2;
      end
    else if (Key=VK_ESCAPE) then begin
      if TextCopy or NewBox then begin
        EPSList.Delete(MNr);
        TextCopy:=false;
        end
      else Orig:=OldOrg;
      end;
    MNr:=-1; chg:=true; NewBox:=false;
    ShowFrame (FramePos,pmXor);
    GrafikFeld.Invalidate;
    end;
  end;

procedure TPrepsForm.FootPosItemClick(Sender: TObject);
var
  h,w : double;
begin
  with PapSize do if Querformat then begin
    w:=y; h:=x;
    end
  else begin
    w:=x; h:=y;
    end;
  if InputDFloat (CursorPos,'Position der Funote','Abstand von links:','cm',
                  'Abstand von unten:','cm','Abbrechen',
                  ffFixed,5,1,5,1,0.2,w-5.0,0.2,h-0.5,FootPosX,FootPosY) then begin
    with FootBox do begin
      Orig.X:=FootPosX;
      Orig.Y:=FootPosY;
      end;
    if EPSList.Count>0 then chg:=true;
    GrafikFeld.Invalidate;
    end;
  end;

procedure TPrepsForm.PrintFootItemClick(Sender: TObject);
begin
  with PrintFootItem do begin
    Checked:=not Checked;
    FootNote:=Checked;
    end;
  if EPSList.Count>0 then chg:=true;
  GrafikFeld.Invalidate;
  end;

procedure TPrepsForm.ChangePaperSize (AIndex : TPaperFormat);

begin
  SetPaperSize(AIndex);
  InitScale(PapSize);
  ResizeImage;
  end;

procedure TPrepsForm.A4FormItemClick(Sender: TObject);
begin
  ChangePaperSize(piA4);
  end;

procedure TPrepsForm.A3FormItemClick(Sender: TObject);
begin
  ChangePaperSize(piA3);
  end;

procedure TPrepsForm.A2FormItemClick(Sender: TObject);
begin
  ChangePaperSize(piA2);
  end;

procedure TPrepsForm.A1FormItemClick(Sender: TObject);
begin
  ChangePaperSize(piA1);
  end;

procedure TPrepsForm.A0FormItemClick(Sender: TObject);
begin
  ChangePaperSize(piA0);
  end;

procedure TPrepsForm.IndiItemClick(Sender: TObject);
begin
  if SizeDialog.Execute (ProgVers,'Papiergre',UserSize) then begin
    ChangePaperSize(piA4);
    ChangePaperSize(piUser);
    end;
  end;

procedure TPrepsForm.TBTopItemClick(Sender: TObject);
begin
  TBTopItem.Checked:=true;
  SetToolbar(false);
  end;

procedure TPrepsForm.TBLeftItemClick(Sender: TObject);
begin
  TBLeftItem.Checked:=true;
  SetToolbar(true);
  end;

end.
