(* Delphi Dialog
   Select Postscript Font
   ==============================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2007
   *)

unit PSFontDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Dialogs, WinPlot;

type
  TFontType = record
    WinName : string;
    PSName  : array [tsNormal..tsBoldItalic] of string[29];
    end;

const
  FontList : array [0..3] of TFontType =
    ((WinName:'Times New Roman';PSName:('Times-Roman','Times-Italic','Times-Bold','Times-BolsItalic')),
     (WinName:'Arial';PSName:('Helvetica','Helvetica-Oblique','Helvetica-Bold','Helvetica-BoldOblique')),
     (WinName:'Courier New';PSName:('Courier','Courier-Oblique','Courier-Bold','Courier-BoldOblique')),
     (WinName:'Symbol';PSName:('Symbol','Symbol','Symbol','Symbol')));

type
  TPSFont = record
    PSFontTyp,
    PSSize,
    PSAngle,
    PSStyle : word;
    PSColor : TColor;
    end;

  TPSFontDialog = class(TForm)
    FontRG: TRadioGroup;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    SizeCB: TComboBox;
    Label3: TLabel;
    ColorBtn: TSpeedButton;
    BoldCB: TCheckBox;
    ItalCB: TCheckBox;
    FarbDialog: TColorDialog;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure ColorBtnClick(Sender: TObject);
    procedure SizeCBChange(Sender: TObject);
  private
    { Private declarations }
    BitMap   : TBitMap;
    PSFont   : TPSFont;
    procedure PlotBitmap (BMWidth,BMHeight : integer;
                          Farbe            : TColor);
  public
    { Public declarations }
    function Execute(APos : TPoint; var APSFont : TPSFont) : boolean;
  end;

var
  PSFontDialog: TPSFontDialog;

implementation

{$R *.DFM}
uses WinUtils;

const
  BitMapHeight=16;
  BitMapWidth=76;

procedure TPSFontDialog.FormCreate(Sender: TObject);
begin
  if not Assigned(Bitmap) then BitMap:=TBitMap.Create;
  with BitMap do begin
    Height:=BitMapHeight; Width:=BitMapWidth;
    end;
  end;
  
{ ------------------------------------------------------------------- }
(* Zeichnen einer neuen Bitmap *)
procedure TPSFontDialog.PlotBitmap (BMWidth,BMHeight : integer;
                                    Farbe            : TColor);
begin
  with BitMap.Canvas do begin
    Brush.Color:=clWhite;
    Brush.Style:=bsSolid;
    FillRect(Rect(0,0,BMWidth-1,BMHeight-1));
    Brush.Color:=Farbe; Pen.Color:=clBlack;
    Rectangle(2,2,BMWidth-3,BMHeight-3);
    end;
  end;

procedure TPSFontDialog.ColorBtnClick(Sender: TObject);
begin
  with FarbDialog do begin
    Color:=PsFont.PSColor;
    if Execute then begin
      PsFont.PSColor:=Color;
      PlotBitMap (BitMapWidth,BitMapHeight,Color);
      ColorBtn.Glyph:=BitMap;
      end;
    end;
  end;

function TPSFontDialog.Execute(APos : TPoint; var APSFont : TPSFont) : boolean;
begin
  with APos do begin
    if (Y < 0) or (X < 0) then Position:=poScreenCenter
    else begin
      Position:=poDesigned;
      if X<0 then X:=Left;
      if Y<0 then Y:=Top;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    end;
  PSFont:=APSFont;
  with PSFont do begin
    FontRG.ItemIndex:=PSFontTyp;
    case PSStyle of
    tsNormal : begin
               BoldCB.Checked:=false; ItalCB.Checked:=false;
               end;
    tsItalic : begin
               BoldCB.Checked:=false; ItalCB.Checked:=true;
               end;
    tsBold   : begin
               BoldCB.Checked:=true; ItalCB.Checked:=false;
               end;
    tsBoldItalic : begin
               BoldCB.Checked:=true; ItalCB.Checked:=true;
               end;
      end;
    SizeCB.Text:=IntToStr (PSSize);
    PlotBitMap (BitMapWidth,BitMapHeight,PSColor);
    ColorBtn.Glyph:=BitMap;
    end;
  if ShowModal=mrOK then with PSFont do begin
    PSFontTyp:=FontRG.ItemIndex;
    if BoldCB.Checked then begin
      if ItalCB.Checked then PSStyle:=tsBoldItalic else PSStyle:=tsBold;
      end
    else if ItalCB.Checked then PSStyle:=tsItalic else PSStyle:=tsNormal;
    APSFont:=PSFont;
    Result:=true;
    end
  else Result:=false;  
  end;

procedure TPSFontDialog.SizeCBChange(Sender: TObject);
var
  n : integer;
begin
  try
    n:=StrToInt(SizeCB.Text);
    PSFont.PSSize:=n;
  except
    on EConvertError do begin
      MessageBeep(0);
      SizeCB.Text:=IntToStr (PSFont.PSSize);
      end;
    end;
  end;

end.
