(* Delphi Dialog
   Input float numbers
   ===================
   possible settings: min and max value, format, digits, units

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 2 - Jul. 2007
   last modified: Nov. 2021
   *)

unit InpDFloat;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, NumberEd, Vcl.StdCtrls,
  Vcl.Buttons;

type
  TInputDFloatDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Desc1Label: TLabel;
    Float1Edit: TFloatRangeEdit;
    Unit1Label: TLabel;
    Float2Edit: TFloatRangeEdit;
    Desc2Label: TLabel;
    Unit2Label: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

(* show dialog and get value *)
function InputDFloat(APos              : TPoint;
                    const Titel,Desc1,AUnits1,
                    Desc2,AUnits2,ACancel : string;
                    AFormat          : TFloatFormat;
                    ADigits1,
                    ADecimal1,
                    ADigits2,
                    ADecimal2        : integer;
                    Min1,Max1,
                    Min2,Max2        : double;
                    var Val1,Val2    : double) : boolean;

var
  InputDFloatDialog : TInputDFloatDialog;

implementation

{$R *.DFM}

uses GgtDummy, WinUtils, NumberUtils;

{ ------------------------------------------------------------------- }
procedure TInputDFloatDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{ ------------------------------------------------------------------- }
procedure TInputDFloatDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s  : string;
  ok : boolean;
  p  : TPoint;
begin
  ok:=CanClose;
  with Float1Edit do begin
    if (ModalResult=mrOK) and RangeError then begin
      if FloatFormat=ffNormalized then
        s:=FloatToStrE(MinValue,Digits)+' .. '+FloatToStrE(MaxValue,Digits)+')'
      else s:=FloatToStrF(MinValue,TFloatFormat(FloatFormat),Digits,Decimal)+' .. '+
         FloatToStrF(MaxValue,TFloatFormat(FloatFormat),Digits,Decimal)+')';
      with ClientOrigin do p:=Point(X+Width,Y+Height);
      CanClose:=false;
      end;
    end;
  if ok then with Float1Edit do begin
    if (ModalResult=mrOK) and RangeError then begin
      if FloatFormat=ffNormalized then
        s:=FloatToStrE(MinValue,Digits)+' .. '+FloatToStrE(MaxValue,Digits)+')'
      else s:=FloatToStrF(MinValue,TFloatFormat(FloatFormat),Digits,Decimal)+' .. '+
         FloatToStrF(MaxValue,TFloatFormat(FloatFormat),Digits,Decimal)+')';
      with ClientOrigin do p:=Point(X+Width,Y+Height);
      CanClose:=false;
      end;
    end;
  CanClose:=ok;
  if not ok then ErrorDialog(p,Format(dgettext('dialogs','Fehler: Bereichsberschreitung (%s)'),[s]));
  end;

{ ------------------------------------------------------------------- }
(* show dialog and get value *)
function InputDFloat(APos              : TPoint;
                    const Titel,Desc1,AUnits1,
                    Desc2,AUnits2,ACancel : string;
                    AFormat          : TFloatFormat;
                    ADigits1,
                    ADecimal1,
                    ADigits2,
                    ADecimal2        : integer;
                    Min1,Max1,
                    Min2,Max2        : double;
                    var Val1,Val2    : double) : boolean;
begin
  if not assigned(InputDFloatDialog) then
    InputDFloatDialog:=TInputDFloatDialog.Create(Application);
  AdjustFormPosition(Screen,InputDFloatDialog,APos);
  with InputDFloatDialog do begin
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    Desc1Label.Caption:=Desc1;
    Unit1Label.Caption:=AUnits1;
    with Float1Edit do begin
      FloatFormat:=TNumFloatFormat(AFormat);
      Digits:=ADigits1; Decimal:=ADecimal1;
      MinValue:=DecimalRound(Min1,ADecimal1); MaxValue:=DecimalRound(Max1,ADecimal1);
      RangeCheck:=false;
      Value:=Val1;
      end;
    Desc2Label.Caption:=Desc2;
    Unit2Label.Caption:=AUnits2;
    with Float2Edit do begin
      FloatFormat:=TNumFloatFormat(AFormat);
      Digits:=ADigits2; Decimal:=ADecimal2;
      MinValue:=DecimalRound(Min2,ADecimal2); MaxValue:=DecimalRound(Max2,ADecimal2);
      RangeCheck:=false;
      Value:=Val2;
      end;
    ActiveControl:=Float1Edit;
    if ShowModal=mrOK then begin
      Val1:=Float1Edit.Value;
      Val2:=Float2Edit.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputDFloatDialog:=nil;
  end;

end.
